==> mp3plot-0.5.1.tar.gz: mp3plot-0.5.1/src/plotters/image_plotter.h <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: image_plotter.h 1702 2009-04-24 20:18:17Z  $

/*!
 * \file
 * Special macros:
 * DISABLE_GRAPHICS: If defined no graphics support will be compiled in
 * \todo TODO: Allow run-time definition of width and height
 */

#ifndef MP3PLOT_IMAGE_PLOTTER_H
#define MP3PLOT_IMAGE_PLOTTER_H

#include <string>
#include <algorithm>

#include "../spinner.h"

#include "../mp3_file.h"
#include "../utils/algo.hxx"
#include "../utils/str_utils.h"
#include "bitrate_plotter.h"
#include "graphics_lib_interface.h"
#include "colour.h"

#include <iostream>

#ifndef DISABLE_GRAPHICS

namespace mp3plot {

    namespace str = net_outlyer::str;
    using net_outlyer::rgb_type;
    using mp3::standard_bitrates;
    using net_outlyer::second_is_less;
    using net_outlyer::spinner;

    //! Plots to an image
    class image_plotter : public bitrate_plotter {
    public:
        virtual const plotter_features& features() const {
            static const plotter_features & f = plotter_features::graphical() |
                                                plotter_features::file();
            return f;
        }
    protected:
//        virtual const plotter_description & description() const = 0;

        ///////////////////////////////////////////////////////////////////
        //   Plotting Algorithm                                          //
        //                                                               //
        //   The plotting process is abstracted, graphical plot          //
        //   implementations must provide a (relatively thin)            //
        //   grapics_lib_interface class, and pass it to plot along the  //
        //   frame information.                                          //
        ///////////////////////////////////////////////////////////////////

        /*!
         * \brief Generic algorithm for the creation of the graphical plot.
         * Implementations must provide a a graphics_lib_interface which wraps the
         * library specific functions.
         */
        template<class ImageType, class ColourType>
        ImageType plot(const frame_collection & frames,
                       const mp3plot::graphics_lib_interface<ImageType, ColourType> & gli,
                       const plot_params& params) const {
            using std::for_each;
            using std::map;
            using str::to_string;

            const float MARGIN_FACTOR = 0.02f;

            const unsigned int CB_WIDTH = frames.size();
            const unsigned int CB_MARGIN = static_cast<int>( (CB_WIDTH * MARGIN_FACTOR) );
            const unsigned int CB_HEIGHT = 50;
            const unsigned int HEIGHT = IMGH - CB_HEIGHT;

            ImageType im = gli.create_image(IMGW, HEIGHT); // "histogram"
            map<bitrate_t, ColourType> hcmap, ccmap;
            ImageType cbIm = gli.create_image(CB_WIDTH+CB_MARGIN*2, CB_HEIGHT); // "colorband"

            // Shorthands
            typedef standard_bitrates::bitrate_vector brvector;
            const brvector & brv = standard_bitrates::as_vector();
            const bitrate_t BAD_BR = standard_bitrates::bad_bitrate();

            typedef map<bitrate_t, int> histogram_values_t;
            histogram_values_t histvalues;

            // Colour allocation & histogram initialisation
            for (brvector::const_iterator it = brv.begin();
                 it != brv.end();
                 ++it)
            {
                const bitrate_t & br = *it;

                // Allocate the colour
                ccmap[br] = gli.allocate_colour(cbIm, bitrate_colour(br, params.old_colourscheme));
                hcmap[br] = gli.allocate_colour(im, bitrate_colour(br, params.old_colourscheme));

                // Initialise all histogram fields
                histvalues[br] = 0;
            }

            // ------------------------------- // FIXME: USE bitrate_distribution!
            // ---------- Histogram ---------- //
            // ------------------------------- //
            for (frame_collection::const_iterator it = frames.begin();
                 it != frames.end();
                 ++it)
            {
                histvalues[it->bitrate()]++;
            }
            int max = std::max_element(histvalues.begin(),
                                       histvalues.end(),
                                       second_is_less<histogram_values_t>())->second;

            const unsigned int MARGIN = static_cast<int>(IMGW * MARGIN_FACTOR);
            const unsigned int MAX_HEIGHT = HEIGHT - MARGIN*2;
            ColourType black = gli.allocate_colour(im, rgb_type(0));
            // Draw OX axis
            gli.draw_line(im, MARGIN, HEIGHT-MARGIN, IMGW-MARGIN, HEIGHT-MARGIN, black);
            // Draw OY axis
            gli.draw_line(im, MARGIN, MARGIN, MARGIN, HEIGHT-MARGIN, black);
            const unsigned int BAR_WIDTH = (IMGW-(MARGIN*2)) / histvalues.size();

            unsigned int xoffset = MARGIN;
            for (histogram_values_t::const_iterator it = histvalues.begin();
                 it != histvalues.end();
                 ++it)
            {
                const bitrate_t & br = it->first;
                // const int height = ( it->second * MAX_HEIGHT ) / frames.size(); // vanilla histogram
                const int height = ( it->second * MAX_HEIGHT ) / max; // scaled height
                const unsigned int labelx = xoffset + (BAR_WIDTH/2);
                assert( labelx < IMGW );
                std::string lbl;
                if (BAD_BR == br) {
                    lbl = "bad";
                }
                else if (0 == br) {
                    lbl = "free";
                }
                else {
                    lbl = str::to_string(br)+"k";
                }
                gli.draw_text(im, labelx, HEIGHT-MARGIN+MARGIN/2, black, lbl);//FIXME coords
                //gli.draw_rectangle(im, xoffset, HEIGHT-MARGIN, xoffset+BAR_WIDTH, HEIGHT-MARGIN-height, hcmap[br]);
                // NOTE gd on Windows doesn't like the heights interchanged (no problems on Linux)
                gli.draw_rectangle(im, xoffset, HEIGHT-MARGIN-height, xoffset+BAR_WIDTH, HEIGHT-MARGIN, hcmap[br]);

                xoffset += BAR_WIDTH;
            }

            // ------------------------------- //
            // --------- Colourband ---------- //
            // ------------------------------- //
            // A line is drawn for each frame // FIXME: group writes
            spinner spn;
            spn.start();

            unsigned int cbxoffset = CB_MARGIN; // vertical line

            assert(CB_HEIGHT > (MARGIN/2));
            for (frame_collection::const_iterator it = frames.begin();
                 it != frames.end();
                 ++it)
            {
                assert( cbxoffset < (CB_WIDTH+CB_MARGIN*2) );

                gli.draw_line(cbIm, cbxoffset, 0, cbxoffset, CB_HEIGHT-MARGIN/2, ccmap[it->bitrate()]);
                ++cbxoffset;
                ++spn;
            }

            // Resize to a better width:
            gli.resize(cbIm, IMGW, CB_HEIGHT);
            spn.stop();

            return gli.vertical_merge(im, cbIm);
        } // plot <>

    // image_plotter's private
    private:
        /*!
         * \brief Image width.
         * This is the final width.
         */
        static const unsigned int IMGW,
        /*!
         * \brief Image height.
         * This is the final height *of each* component, but there are
         * two components: the plot itself and the legend, so the output
         * image will have a height of \p IMGH*2
         */
                                 IMGH;

        /*!
         * \brief Returns the colour corresponding to a bitrate.
         */
        static rgb_type bitrate_colour(bitrate_t, bool use_old_colourscheme=false);
    }; // image_plotter

} // namespace mp3plot

#endif // DISABLE_GRAPHICS

#endif // MP3PLOT_IMAGE_PLOTTER_H
// vim:set ts=4 et ai:

==> mp3plot-0.6.0.tar.gz: mp3plot-0.6.0/src/plotters/image_plotter.h <==
/*
 * mp3plot - Bitrate analysis tool
 *
 * Copyright (C) 2007, 2009 Toni Corvera
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// $Id: image_plotter.h 1342 2009-06-21 23:41:45Z toni $

/*!
 * \file
 * Special macros:
 * DISABLE_GRAPHICS: If defined no graphics support will be compiled in
 * \todo TODO: Allow run-time definition of width and height
 */

#ifndef MP3PLOT_IMAGE_PLOTTER_H
#define MP3PLOT_IMAGE_PLOTTER_H

#include <string>
#include <algorithm>

#include "../spinner.h"

#include "../mp3_file.h"
#include "../utils/algo.hxx"
#include "../utils/str_utils.h"
#include "bitrate_plotter.h"
#include "graphics_lib_interface.h"
#include "colour.h"

#ifndef DISABLE_GRAPHICS

namespace mp3plot {

    namespace str = net_outlyer::str;
    using net_outlyer::rgb_type;
    using net_outlyer::second_is_less;
    using net_outlyer::spinner;

    //! Plots to an image
    class image_plotter : public bitrate_plotter {
    public:
        virtual const plotter_features& features() const {
            static const plotter_features & f = plotter_features::graphical() |
                                                plotter_features::file();
            return f;
        }
    protected:
//        virtual const plotter_description & description() const = 0;

        ///////////////////////////////////////////////////////////////////
        //   Plotting Algorithm                                          //
        //                                                               //
        //   The plotting process is abstracted, graphical plot          //
        //   implementations must provide a (relatively thin)            //
        //   grapics_lib_interface class, and pass it to plot along the  //
        //   frame information.                                          //
        ///////////////////////////////////////////////////////////////////

        /*!
         * \brief Generic algorithm for the creation of the graphical plot.
         * Implementations must provide a a graphics_lib_interface which wraps the
         * library specific functions.
         */
        template<class ImageType, class ColourType>
        ImageType plot(const frame_collection & frames,
                       const mp3plot::graphics_lib_interface<ImageType, ColourType> & gli,
                       const plot_params& params) const {
            using std::for_each;
            using std::map;
            using str::to_string;

            using mp3::bitrate_translator;

            const float MARGIN_FACTOR = 0.02f;

            const unsigned int CB_WIDTH = frames.size();
            const unsigned int CB_MARGIN = static_cast<int>( (CB_WIDTH * MARGIN_FACTOR) );
            const unsigned int CB_HEIGHT = 50;
            const unsigned int HEIGHT = IMGH - CB_HEIGHT;

            ImageType im = gli.create_image(IMGW, HEIGHT); // "histogram"
            map<bitrate_t, ColourType> hcmap, ccmap;
            ImageType cbIm = gli.create_image(CB_WIDTH+CB_MARGIN*2, CB_HEIGHT); // "colorband"

#if 0 // TODO: Move back to iterators
            // Shorthands
            typedef standard_bitrates::bitrate_vector brvector;
            const brvector & brv = standard_bitrates::as_vector();
            const bitrate_t BAD_BR = standard_bitrates::bad_bitrate();

            typedef map<bitrate_t, int> histogram_values_t;
            histogram_values_t histvalues;

            // Colour allocation & histogram initialisation
            for (brvector::const_iterator it = brv.begin();
                 it != brv.end();
                 ++it)
            {
                const bitrate_t & br = *it;

                // Allocate the colour
                ccmap[br] = gli.allocate_colour(cbIm, bitrate_colour(br, params.old_colourscheme));
                hcmap[br] = gli.allocate_colour(im, bitrate_colour(br, params.old_colourscheme));

                // Initialise all histogram fields
                histvalues[br] = 0;
            }
#endif
            const bitrate_t BAD_BR = bitrate_translator::bad_bitrate();
            typedef map<bitrate_t, int> histogram_values_t;
            histogram_values_t histvalues;

            // TODO: Register changes in version (layer?) from frame to frame
            for (uint8_t bridx=0; bridx<bitrate_translator::number(); ++bridx) {
                // Allocate the colour
                ccmap[bridx] = gli.allocate_colour(cbIm, bitrate_colour(bridx, params.old_colourscheme));
                hcmap[bridx] = gli.allocate_colour(im, bitrate_colour(bridx, params.old_colourscheme));
            }

            // ------------------------------- //
            // ---------- Histogram ---------- //
            // ------------------------------- //
            const bitrate_distribution & dist = params.br_dist;

            const unsigned int MARGIN = static_cast<int>(IMGW * MARGIN_FACTOR);
            const unsigned int MAX_HEIGHT = HEIGHT - MARGIN*2;
            ColourType black = gli.allocate_colour(im, rgb_type(0));
            // Draw OX axis
            gli.draw_line(im, MARGIN, HEIGHT-MARGIN, IMGW-MARGIN, HEIGHT-MARGIN, black);
            // Draw OY axis
            gli.draw_line(im, MARGIN, MARGIN, MARGIN, HEIGHT-MARGIN, black);

            const unsigned int BAR_WIDTH = (IMGW-(MARGIN*2)) / 15; // (dist.bitrates_number()-1);

            unsigned int xoffset = MARGIN;

            const mp3::frame & frm = *frames.begin();
            const mpeg_version_t ver = frm.get_mpeg_version();
            const mpeg_layer_t lyr = frm.get_mpeg_layer();
            for (bitrate_distribution::const_iterator it = dist.begin(); it != dist.end(); ++it)
            {
                uint8_t bidx = it.bitrate_index();
                const bitrate_t & br = bitrate_translator::get(bidx, ver, lyr);
                // const int height = ( it->second * MAX_HEIGHT ) / frames.size(); // vanilla histogram
                const int height = ( it.numframes() * MAX_HEIGHT ) / dist.max; // scaled height
                const unsigned int labelx = xoffset + (BAR_WIDTH/2);
                assert( labelx < IMGW );
                std::string lbl;
                if (BAD_BR == br) {
                    lbl = "bad";
                }
                else if (0 == br) {
                    lbl = "free";
                }
                else {
                    lbl = str::to_string(br)+"k";
                }
                gli.draw_text(im, labelx, HEIGHT-MARGIN+MARGIN/2, black, lbl);//FIXME coords
                //gli.draw_rectangle(im, xoffset, HEIGHT-MARGIN, xoffset+BAR_WIDTH, HEIGHT-MARGIN-height, hcmap[br]);
                // NOTE gd on Windows doesn't like the heights interchanged (no problems on Linux)
                gli.draw_rectangle(im, xoffset, HEIGHT-MARGIN-height, xoffset+BAR_WIDTH, HEIGHT-MARGIN, hcmap[bidx]);

                xoffset += BAR_WIDTH;
            }


            // ------------------------------- //
            // --------- Colourband ---------- //
            // ------------------------------- //
            // A line is drawn for each frame // FIXME: group writes
            spinner spn;
            spn.start();

            unsigned int cbxoffset = CB_MARGIN; // vertical line

            assert(CB_HEIGHT > (MARGIN/2));
            for (frame_collection::const_iterator it = frames.begin();
                 it != frames.end();
                 ++it)
            {
                assert( cbxoffset < (CB_WIDTH+CB_MARGIN*2) );

                gli.draw_line(cbIm, cbxoffset, 0, cbxoffset, CB_HEIGHT-MARGIN/2, ccmap[it->get_bitrate_index()]);
                ++cbxoffset;
                ++spn;
            }

            // Resize to a better width:
            gli.resize(cbIm, IMGW, CB_HEIGHT);
            spn.stop();

            return gli.vertical_merge(im, cbIm);
        } // plot <>

    // image_plotter's private
    private:
        /*!
         * \brief Image width.
         * This is the final width.
         */
        static const unsigned int IMGW,
        /*!
         * \brief Image height.
         * This is the final height *of each* component, but there are
         * two components: the plot itself and the legend, so the output
         * image will have a height of \p IMGH*2
         */
                                 IMGH;

        /*!
         * \brief Returns the colour corresponding to a bitrate.
         */
        static rgb_type bitrate_colour(uint8_t, bool use_old_colourscheme=false);
    }; // image_plotter

} // namespace mp3plot

#endif // DISABLE_GRAPHICS

#endif // MP3PLOT_IMAGE_PLOTTER_H
// vim:set ts=4 et ai:
